Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

// Gecko 1.9.0/1.9.1 compatibility - add XPCOMUtils.defineLazyServiceGetter
if (!("defineLazyServiceGetter" in XPCOMUtils))
{
  XPCOMUtils.defineLazyServiceGetter = function XPCU_defineLazyServiceGetter(obj, prop, contract, iface)
  {
    obj.__defineGetter__(prop, function XPCU_serviceGetter()
    {
      delete obj[prop];
      return obj[prop] = Components.classes[contract].getService(Components.interfaces[iface]);
    });
  };
}

function StylishStartup() {
}
StylishStartup.prototype = {
  classID: Components.ID("{a0029965-c87c-486b-b721-94c965622004}"),
  contractID: "@stylish/startup;1",
  classDescription: "Stylish Startup",

  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsISupports,
                                         Components.interfaces.nsIObserver]),

  observe: function(aSubject, aTopic, aData) {
    switch(aTopic) {
      case "xpcom-startup":
        // this is run very early, right after XPCOM is initialized, but before
        // user profile information is applied. Register ourselves as an observer
        // for 'profile-after-change' and 'quit-application'.
        var obsSvc = Components.classes["@mozilla.org/observer-service;1"]
                       .getService(Components.interfaces.nsIObserverService);
        //obsSvc.addObserver(this, "profile-after-change", false);
        obsSvc.addObserver(this, "quit-application", false);
        //break;

      case "profile-after-change":
        // This happens after profile has been loaded and user preferences have been read.
        // startup code here
        var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
                            .getService(Components.interfaces.nsIStyleSheetService);
        var ios = Components.classes["@mozilla.org/network/io-service;1"]
                            .getService(Components.interfaces.nsIIOService);
        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
        var ds = rdfService.GetDataSourceBlocking(this.getDatasourceURI());
        var rdfContainerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].getService(Components.interfaces.nsIRDFContainerUtils);
        var container = rdfContainerUtils.MakeSeq(ds, rdfService.GetResource("urn:stylish:userstyles"));
        var children = container.GetElements();
        var enabledPredicate = rdfService.GetResource("urn:stylish#enabled");
        var codePredicate = rdfService.GetResource("urn:stylish#code");
        while (children.hasMoreElements()) {
          var current = children.getNext();
          var enabledNode = ds.GetTarget(current, enabledPredicate, true)
          if (enabledNode) {
            var enabled = enabledNode.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            if (enabled == "true") {
              var cssNode = ds.GetTarget(current, codePredicate, true);
              if (cssNode) {
                var css = cssNode.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
                var u = ios.newURI("data:text/css," + css, null, null);
                sss.loadAndRegisterSheet(u, sss.USER_SHEET);
              }
            }
          }
        }
        break;

      case "quit-application":
        // shutdown code here
        break;

      default:
        throw Components.Exception("Unknown topic: " + aTopic);
    }
  },

  get prefService() {
    delete this.prefService;
    return this.prefService = Services.prefs;
  },

  getDatasourceURI: function() {
    var prefPath = this.prefService.getCharPref("extensions.stylish.fileURL");
    if (prefPath.length > 0) {
      return prefPath;
    }

    var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    file.append("stylish.rdf");
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    if (!file.exists()) {
      //either this is the first run or the user deleted his file (the bastard)

      //read the default file's contents (courtesy Torisugari <http://forums.mozillazine.org/viewtopic.php?p=921150#921150>)
      var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"].getService(Components.interfaces.nsIScriptableInputStream);
      var channel = ioService.newChannel("chrome://stylish/content/stylish-default.rdf", null, null);
      var input = channel.open();
      scriptableStream.init(input);
      var data = scriptableStream.read(input.available());
      scriptableStream.close();
      input.close();

      //write the contents to the profile file
      var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
      foStream.init(file, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
      foStream.write(data, data.length);
      foStream.close();
    }
    return ioService.newFileURI(file).spec;
  }
};

// The following line is what XPCOM uses to create components. Each component prototype
// must have a .classID which is used to create it.
/**
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
*/
if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([StylishStartup]);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule([StylishStartup]);
